/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSDOCUMENT_H
#define KBSDOCUMENT_H

#include <QHash>
#include <QList>
#include <QSet>
#include <QStringList>
#include <QWidget>

#include <KDE/KConfigSkeleton>
#include <KDE/KSharedConfig>
#include <KDE/KUrl>

#include <kbscore_export.h>
#include <kbspreferences.h>
#include <kbstree.h>

class KBSBOINCMonitor;
class KBSLocation;
class KBSProjectMonitor;
class KBSProjectPlugin;
class KBSTaskMonitor;

class KBSCORE_EXPORT KBSDocument : public KBSTreeNode
{
  Q_OBJECT
  public:
    KBSDocument(QObject *parent=0);
    virtual ~KBSDocument();
    
    virtual void connectTo(const KBSLocation &location);
    virtual void disconnectFrom(const KBSLocation &location);
    
    virtual int interval() const;
    
    virtual QSet<KBSProjectPlugin*> plugins() const;
    virtual KBSProjectPlugin *plugin(const QString &project) const;
    
    virtual KConfigSkeleton *preferences();
    
    virtual void readConfig(KSharedConfigPtr config);
    virtual void writeConfig(KSharedConfigPtr config);
  
  public slots:
    virtual void applyPreferences();
  
  signals:
    void intervalChanged(int interval);
  
  private:
    void loadPlugins();
  
  protected:
    QHash<KUrl,KBSLocation> m_locations;
    QHash<QString,KBSProjectPlugin*> m_plugins;
    KBSPreferences m_preferences;
};

#endif
