/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSEINSTEINDATA_H
#define KBSEINSTEINDATA_H

#include <QDateTime>
#include <QHash>
#include <QList>
#include <QString>

#include <KDE/KUrl>

#include <kbseinstein_export.h>

namespace KBSEinstein
{
  KBSEINSTEIN_EXPORT QString formatIFOName(const QString &ifo);
  KBSEINSTEIN_EXPORT KUrl formatIFOURL(const QString &ifo);
}

typedef QHash<QString,QString> KBSEinsteinAttributeSet;

struct KBSEINSTEIN_EXPORT KBSEinsteinCmdLineArgs
{
  QList<KBSEinsteinAttributeSet> arg;
  
  bool parse(const QString &input);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinConf
{
  KBSEinsteinAttributeSet set;
  
  bool parse(const QStringList &lines);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinCoord
{
  double freq,
         ra,
         dec;
};

struct KBSEINSTEIN_EXPORT KBSEinsteinFstat
{
  KBSEinsteinCoord coord;
  double f1dot;
  double score;
  
  bool parse(const QString &line);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinFstatOut
{
  QList<KBSEinsteinFstat> fstat;
  
  bool parse(const QStringList &lines);
};

struct KBSEINSTEIN_EXPORT KBSAlbertResult
{
  KBSEinsteinCmdLineArgs args;
  KBSEinsteinConf conf;
  KBSEinsteinFstatOut fstat_out;
};

struct KBSEINSTEIN_EXPORT KBSEinsteinMass
{
  QList< QList<double> > data;
  unsigned count;
  double value;
  
  bool parse(const QStringList &lines);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinH
{
  KBSEinsteinCoord coord;
  double score;
  
  bool parse(const QString &line);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinCoincidence
{
  unsigned index[2];
  double coincidence;
  
  bool parse(const QString &line);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinPolkaOut
{
  QList<KBSEinsteinH> h[2];
  QList<KBSEinsteinCoincidence> coincidences;
  
  bool parse(const QStringList &lines);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinResult
{
  KBSEinsteinCmdLineArgs args;
  KBSEinsteinConf conf;
  KBSEinsteinMass earth, sun;
  KBSEinsteinPolkaOut polka_out;
};

struct KBSEINSTEIN_EXPORT KBSEinsteinFstatH
{
  KBSEinsteinCoord coord;
  unsigned count;
  double value[2];
  double score;
  
  bool parse(const QString &line);
};

struct KBSEINSTEIN_EXPORT KBSEinsteinState
{
  QList<KBSEinsteinFstatH> fstats[2];
  
  bool parse(const QStringList &lines, unsigned index, unsigned pos=0);
};

#endif
