/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSEINSTEINPROJECTMONITOR_H
#define KBSEINSTEINPROJECTMONITOR_H

#include <QHash>

#include <kbsprojectmonitor.h>

#include <kbseinstein_export.h>

#include <kbseinsteindata.h>

class KBSEINSTEIN_EXPORT KBSEinsteinProjectMonitor : public KBSProjectMonitor
{
  Q_OBJECT
  public:
    KBSEinsteinProjectMonitor(const QString &project, KBSBOINCMonitor *parent);
    virtual ~KBSEinsteinProjectMonitor();
    
    virtual const KBSAlbertResult *albertResult(const QString &workunit);
    virtual const KBSEinsteinResult *einsteinResult(const QString &workunit);
    
    virtual void setEinsteinState(const QString &workunit, const KBSEinsteinState &state);
    
    virtual KBSLogManager *logManager() const;
  
  protected:
    virtual QString app(const QString &workunit);
    
    virtual bool parseable(const QString &openName) const;
    virtual bool parseFile(KBSFileInfo *file, const QString &fileName);
    
    virtual KBSAlbertResult *mkAlbertResult(const QString &workunit);
    virtual KBSEinsteinResult *mkEinsteinResult(const QString &workunit);
  
  private:
    void setConf(const KBSEinsteinConf &conf, const QStringList &workunits);
    void setEarth(const KBSEinsteinMass &earth, const QStringList &workunits);
    void setSun(const KBSEinsteinMass &sun, const QStringList &workunits);
    void setPolkaOut(const KBSEinsteinPolkaOut &polka_out, const QStringList &workunits);
    void setFstatOut(const KBSEinsteinFstatOut &fstat_out, const QStringList &workunits);
  
  private slots:
    void removeWorkunits(const QStringList &workunits);
    void updateFile(const QString &fileName);
  
  protected:
    QHash<QString,KBSAlbertResult*> m_albert;
    QHash<QString,KBSEinsteinResult*> m_einstein;
  
  private:
    QHash<QString,QString> m_app;
};

#endif
