/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSMOLECULEVIEW_H
#define KBSMOLECULEVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_GL

#include <QDomDocument>
#include <QGLWidget>
#include <QHash>
#include <QKeyEvent>
#include <QMouseEvent>
#include <QPixmap>
#include <QTextStream>

#include <kbsmoleculemodel.h>

class KBSMoleculeView : public QGLWidget, KBSMoleculeRenderer
{
  Q_OBJECT
  public:
    KBSMoleculeView(QWidget *parent=0);
    virtual ~KBSMoleculeView();
    
    virtual void setColor(QColor color);
    virtual void drawBall(double *v, double radius);
    virtual void drawLine(double *v1, double *v2, double thickness=0.0);
    virtual void drawQuadrangle(double *v1, double *v2, double *v3, double *v4);
    
    QString header() const;
    virtual void setHeader(const QString &header);
    
    QString footer() const;
    virtual void setFooter(const QString &footer);
    
    virtual KBSMoleculeModel *model() const;
    
    virtual QPixmap pixmap();
  
  protected:
    virtual void initializeGL();
    virtual void resizeGL(int width, int height);
    virtual void paintGL();
    
    virtual void keyPressEvent(QKeyEvent *e);
    virtual void mousePressEvent(QMouseEvent *e);
    virtual void mouseReleaseEvent(QMouseEvent *e);
    virtual void mouseMoveEvent(QMouseEvent *e);
  
  private:
    void updateTexture(unsigned texture);
    void drawText();
  
  protected:
    double m_scale;
  
  private:
    KBSMoleculeModel *m_model;
    
    bool m_tracking;
    QPoint m_last;
    
    GLUquadricObj *m_quadric;
    enum Shape { Sphere, Cylinder, Shapes };
    GLuint m_base;
    
    enum { Header, Footer, Text };
    QString m_text[Text];
    QSize m_size[Text];
    GLuint *m_texture;
    
    static const unsigned s_border;
    static const unsigned s_fontSize[Text];
};

#else

#include <QWidget>

class KBSMoleculeView : public QWidget
{
  Q_OBJECT
  public:
    KBSMoleculeView(QWidget *parent=0) : QWidget(parent) {}
};

#endif

#endif
