/***************************************************************************
 *   Copyright (C) 2004-2008 by Roberto Virga                              *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSPANELNODE_H
#define KBSPANELNODE_H

#include <QHash>
#include <QStringList>

#include <kbscore_export.h>
#include <kbspanel.h>
#include <kbstree.h>

class KBSBOINCMonitor;
class KBSField;

class KBSCORE_EXPORT KBSPanelNode : public KBSTreeNode
{
  Q_OBJECT
  public:
    KBSPanelNode(KBSTreeNode *parent);
    
    virtual unsigned type() const;
    
    virtual KBSPanel *createPanel(QWidget *parent=0);
    
    virtual QStringList fieldNames() const;
    virtual const KBSField *field(const QString &name) const;
  
  protected:
    virtual KBSBOINCMonitor *monitor();
    
    virtual KBSField *createField(const QString &name);
  
  private slots:
    void slotDestroyed(QObject *panel);
  
  protected:
    QList<KBSPanel*> m_panels;
  
  private:
    KBSBOINCMonitor *m_monitor;
    QStringList m_fieldNames;
    QHash<QString,KBSField*> m_fields;
};

class KBSCORE_EXPORT KBSProjectPanelNode : public KBSPanelNode
{
  public:
    KBSProjectPanelNode(KBSTreeNode *parent, const QString &project);
  
  protected:
    virtual QString project() const;
  
  private:
    QString m_project;
};

class KBSCORE_EXPORT KBSWorkunitPanelNode : public KBSPanelNode
{
  public:
    KBSWorkunitPanelNode(KBSTreeNode *parent, const QString &workunit);
  
  protected:
    virtual QString project() const;
    virtual QString workunit() const;
    virtual QString result() const;
  
  private:
    QString m_project,
            m_workunit,
            m_result;
};

class KBSCORE_EXPORT KBSTaskPanelNode : public KBSPanelNode
{
  public:
    KBSTaskPanelNode(KBSTreeNode *parent, unsigned task);
  
  protected:
    virtual QString project() const;
    virtual QString workunit() const;
    virtual QString result() const;
    virtual unsigned task() const;
  
  private:
    QString m_project,
            m_workunit,
            m_result;
    unsigned m_task;

};

#endif
