/***************************************************************************
 *   Copyright (C) 2005 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSPREDICTORDATA_H
#define KBSPREDICTORDATA_H

#include <QDateTime>
#include <QStringList>

#include <KDE/KUrl>

#include <kbsmolbiodata.h>

#include <kbspredictor_export.h>

namespace KBSPredictor
{
  KBSPREDICTOR_EXPORT unsigned casp(unsigned target);
  
  KBSPREDICTOR_EXPORT unsigned parseCASPTarget(const QString &name);
  KBSPREDICTOR_EXPORT QString formatCASPTargetName(unsigned target);
  KBSPREDICTOR_EXPORT KUrl formatCASPTargetURL(unsigned target);
}

struct KBSPREDICTOR_EXPORT KBSPredictorBurials
{
  double avg[AminoAcids+1][AminoAcids+1];
  double sdev[AminoAcids+1][AminoAcids+1];
  unsigned pairs[AminoAcids+1][AminoAcids+1];
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSPredictorECovers24
{
  double value[AminoAcids][25];
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSPredictorProfile3
{
  double value[AminoAcids][5][5][5];
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSPredictorQuasi3
{
  double par[2][AminoAcids][AminoAcids];
  double mid[2][AminoAcids][AminoAcids];
  double ant[2][AminoAcids][AminoAcids];
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSPredictorScale3B
{
  KBSAminoAcid aa[3];
  unsigned count[3];
  double value;
  
  bool parse(const QString &line);
};

struct KBSPREDICTOR_EXPORT KBSPredictorS1234
{
  double r1_2[AminoAcids][AminoAcids][3];
  double r1_3[AminoAcids][AminoAcids][4];
  double r1_4[AminoAcids][AminoAcids][14];
  double r1_5[AminoAcids][AminoAcids][7];
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSMONSSTERInput
{
  unsigned random,                    // random seed
           ncycle,                    // number of cycles per temperature step
           icycle,                    // Monte Carlo moves per cycle
           tsteps,                    // temperature steps
           resmin,                    // first fragment residue (fixed template)
           resmax;                    // last fragment residue (fixed template)
  double temp[2],                     // temperature range
         softcore,                    // soft core radius for large residues
         central,                     // centrosymmetric potential scaling
         stiff,                       // general stiffness potential scaling
         pair,                        // pair potential scaling,
         kdcore,                      // hydrophobic core potential scaling
         hbond,                       // H-bond cooperativity potential scaling
         shrt,                        // short scale potential scaling
         burial,                      // burial potential scaling
         multibody,                   // multi body potential scaling
         threebody;                   // three body potential scaling
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSMONSSTERResidue
{
  unsigned resSeq;
  KBSAminoAcid resName;
  unsigned count[2];
  
  bool parse(const QString &line);
};

struct KBSPREDICTOR_EXPORT KBSMONSSTERSeq
{
  QList<KBSMONSSTERResidue> groups;
  unsigned atoms;
  
  bool parse(const QStringList &lines);
  
  KBSProteinFASTA toFASTA() const;
  QString toString() const;
};

struct KBSPREDICTOR_EXPORT KBSMONSSTERRestraint
{
  unsigned num;
  double value;
  
  bool parse(const QString &line);
};

struct KBSPREDICTOR_EXPORT KBSPredictorProteinNOE
{
  struct {
    unsigned index;
    QString name;
  } select[2];
  double kmin,
         rmin,
         kmax,
         rmax;
  
  bool parse(const QString &line);
};

struct KBSPREDICTOR_EXPORT KBSMFOLDResult
{
  KBSPredictorBurials burials;
  KBSPredictorECovers24 ecovers_24;
  KBSPredictorProfile3 profile3;
  KBSPredictorQuasi3 quasi3;
  QList<KBSPredictorScale3B> scale3b;
  KBSPredictorS1234 s1234, s1234h, s1234e;
  struct {
    KBSProteinChain init_chain;
    KBSMONSSTERInput input;
    KBSMONSSTERSeq seq;
    QList<KBSMONSSTERRestraint> restraints;
    struct {
      KBSProteinChain chain;
      KBSProteinPDB pdb;
      QList<KBSPredictorProteinNOE> noe;
    } final;
  } monsster;
};

struct KBSPREDICTOR_EXPORT KBSPredictorSeed
{
  unsigned stream;
  
  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSPredictorCHARMMInp
{
  unsigned ntemps,
           nsteps;
  struct {
    unsigned low,
             high;
  } t;

  bool parse(const QStringList &lines);
};

struct KBSPREDICTOR_EXPORT KBSCHARMMResult
{
  KBSPredictorCHARMMInp inp;
  struct {
    KBSProteinPDB pdb, final_pdb;
    QList<KBSPredictorProteinNOE> noe;
    unsigned residuals;
  } protein;
  struct {
    unsigned stream;
  } seed;
};

struct KBSPREDICTOR_EXPORT KBSMONSSTERRestart
{
  unsigned line1a[2];
  double line1b[5];
  unsigned line2[3];
  double line3[2];
  KBSProteinChain chain;
  
  bool parse(const QStringList &lines);
};

#endif
