/***************************************************************************
 *   Copyright (C) 2005 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSPREDICTORPROJECTMONITOR_H
#define KBSPREDICTORPROJECTMONITOR_H

#include <QHash>

#include <kbsprojectmonitor.h>

#include <kbspredictor_export.h>

#include <kbspredictordata.h>

class KBSMoleculeLog;

class KBSPREDICTOR_EXPORT KBSPredictorProjectMonitor : public KBSProjectMonitor
{
  Q_OBJECT
  public:
    KBSPredictorProjectMonitor(const QString &project, KBSBOINCMonitor *parent);
    virtual ~KBSPredictorProjectMonitor();
    
    virtual const KBSMFOLDResult *mfoldResult(const QString &workunit);
    virtual const KBSCHARMMResult *charmmResult(const QString &workunit);

    virtual void setMFOLDState(const QString &workunit, const KBSMONSSTERRestart &state);
    
    virtual KBSLogManager *logManager() const;
    virtual KBSMoleculeLog *moleculeLog() const;
  
  protected:
    virtual bool parseable(const QString &openName) const;
    virtual bool parseFile(KBSFileInfo *file, const QString &fileName);
    
    virtual KBSMFOLDResult *mkMFOLDResult(const QString &workunt);
    virtual KBSCHARMMResult *mkCHARMMResult(const QString &workunt);
  
  private:
    bool parseScale3B(const QStringList &lines, QList<KBSPredictorScale3B> &scale3b);
    bool parseMONSSTERRestraints(const QStringList &lines, QList<KBSMONSSTERRestraint> &restraints);
    bool parseProteinNOE(const QStringList &lines, QList<KBSPredictorProteinNOE> &noe);
    bool parseSeedStream(const QStringList &lines, unsigned &seed);
    
    void setBurials(const KBSPredictorBurials &burials, const QStringList &workunits);
    void setECovers24(const KBSPredictorECovers24 &ecovers_24, const QStringList &workunits);
    void setProfile3(const KBSPredictorProfile3 &profile3, const QStringList &workunits);
    void setQuasi3(const KBSPredictorQuasi3 &quasi3, const QStringList &workunits);
    void setScale3B(const QList<KBSPredictorScale3B> &scale3b, const QStringList &workunits);
    void setS1234(const KBSPredictorS1234 &s1234, const QStringList &workunits);
    void setS1234H(const KBSPredictorS1234 &s1234h, const QStringList &workunits);
    void setS1234E(const KBSPredictorS1234 &s1234e, const QStringList &workunits);
    void setMONSSTERInitChain(const KBSProteinChain &init_chain, const QStringList &workunits);
    void setMONSSTERInput(const KBSMONSSTERInput &input, const QStringList &workunits);
    void setMONSSTERSeq(const KBSMONSSTERSeq &seq, const QStringList &workunits);
    void setMONSSTERRestraints(const QList<KBSMONSSTERRestraint> &restraints, const QStringList &workunits);
    void setMONSSTERFinalChain(const KBSProteinChain &final_chain, const QStringList &workunits);
    void setMONSSTERFinalPDB(const KBSProteinPDB &final_pdb, const QStringList &workunits);
    void setMONSSTERFinalNOE(const QList<KBSPredictorProteinNOE> &final_noe, const QStringList &workunits);
    void setCHARMMInp(const KBSPredictorCHARMMInp &inp, const QStringList &workunits);
    void setProteinPDB(const KBSProteinPDB &pdb, const QStringList &workunits);
    void setProteinNOE(const QList<KBSPredictorProteinNOE> &noe, const QStringList &workunits);
    void setSeedStream(unsigned seed, const QStringList &workunits);
    void setProteinFinalPDB(const KBSProteinPDB &final_pdb, const QStringList &workunits);
    
    static unsigned countGroups(const QList<KBSAtomPDB> &pdb);
  
  private slots:
    void removeWorkunits(const QStringList &workunits);
    void logResults(const QStringList &results);
    void updateFile(const QString &fileName);
  
  protected:
    QHash<QString,KBSMFOLDResult*> m_mfold;
    QHash<QString,KBSCHARMMResult*> m_charmm;
  
  private:
    QStringList m_start;
};

#endif
