/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSPROJECTPLUGIN_H
#define KBSPROJECTPLUGIN_H

#include <QList>
#include <QWidget>

#include <KDE/KConfigSkeleton>
#include <KDE/KSharedConfig>

#include <kbscore_export.h>

class KBSDocument;
class KBSBOINCMonitor;
class KBSProjectMonitor;
class KBSTaskMonitor;

class KBSCORE_EXPORT KBSConfigPage : public QWidget
{
  public:
    virtual ~KBSConfigPage() {}
    
    virtual QString icon() const = 0;
    virtual QString title() const = 0;
    virtual QString description() const = 0;
  
  protected:
    KBSConfigPage(QWidget *parent=0) : QWidget(parent) {}
};

class KBSCORE_EXPORT KBSProjectPlugin : public QObject
{
  Q_OBJECT
  public:
    virtual QString name() const;
    
    virtual QList<KBSConfigPage*> createConfigPages();
    virtual KBSProjectMonitor *createProjectMonitor(const QString &project, KBSBOINCMonitor *parent) = 0;
    virtual KBSTaskMonitor *createTaskMonitor(unsigned task, KBSBOINCMonitor *parent) = 0;
     
    virtual KConfigSkeleton *preferences();
    virtual void applyPreferences();
    
    virtual void readConfig(KSharedConfigPtr config) = 0;
    virtual void writeConfig(KSharedConfigPtr config) = 0;
  
  protected:
    KBSProjectPlugin(KBSDocument *parent, const QString &name);
    
    KBSDocument *document() const;
};

#endif
