/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSROSETTADATA_H
#define KBSROSETTADATA_H

#include <QHash>
#include <QStringList>

#include <kbsmolbiodata.h>

struct KBSRosettaPsiPredAtom {
  KBSAminoAcid aa;
  KBSFeature feature;
  double x, y, z;
  
  bool parse(const QString &line);
};

struct KBSRosettaPsiPredSS2 {
  QList<KBSRosettaPsiPredAtom> sequence;
  
  KBSProteinFASTA toFASTA() const;
  
  bool parse(const QStringList &lines);
};

struct KBSRosettaNeighborAtom {
  KBSAminoAcid aa;
  KBSFeature feature;
  double score[3];
  
  bool parse(const QString &line, QString *name=0, char *type=0, unsigned *start=0);
};

struct KBSRosettaNeighbor {
  char type;
  QString name;
  unsigned start;
  QList<KBSRosettaNeighborAtom> sequence;
};

struct KBSRosettaNeighborGroup {
  QList <KBSRosettaNeighbor> neighbor;
};

struct KBSRosettaV13 {
  QList<KBSRosettaNeighborGroup> group;
  
  bool parse(const QStringList &lines, unsigned pos=0);
};

struct KBSRosettaAcceptedAtom
{
  KBSFeature feature;
  double score[3],
         x,
         y,
         z;
  
  bool parse(const QString &description, const QString &line);
};

struct KBSRosettaAccepted {
  QString description;
  QHash<QString, double> score;
  QList<KBSRosettaAcceptedAtom> sequence;
  
  KBSProteinChain toChain() const;
};

struct KBSRosettaOut {
  QStringList keys;
  QList<KBSAminoAcid> sequence;
  QList<KBSRosettaAccepted> accepted;
  
  KBSProteinFASTA toFASTA() const;
  
  bool parse(const QStringList &lines, unsigned pos=0);
};

struct KBSRosettaResult {
  KBSProteinPDB native;
  KBSProteinFASTA fasta;
  KBSRosettaPsiPredSS2 psipred_ss2;
  KBSRosettaV13 v1_3[2];
  KBSRosettaOut out;
};

#endif
