/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSROSETTAPROJECTMONITOR_H
#define KBSROSETTAPROJECTMONITOR_H

#include <QHash>

#include <kbsprojectmonitor.h>

#include <kbsrosetta_export.h>

#include <kbsrosettadata.h>

class KBSMoleculeLog;

class KBSROSETTA_EXPORT KBSRosettaProjectMonitor : public KBSProjectMonitor
{
  Q_OBJECT
  public:
    KBSRosettaProjectMonitor(const QString &project, KBSBOINCMonitor *parent);
    virtual ~KBSRosettaProjectMonitor();
    
    virtual bool hasNative(const QString &workunit) const;
    virtual const KBSRosettaResult *rosettaResult(const QString &workunit);
    
    virtual void setRosettaState(const QString &workunit, const KBSRosettaOut &state);
    
    virtual KBSLogManager *logManager() const;
    virtual KBSMoleculeLog *moleculeLog() const;
  
  protected:
    virtual bool parseable(const QString &openName) const;
    virtual bool parseFile(KBSFileInfo *file, const QString &fileName);
    
    virtual KBSRosettaResult *mkRosettaResult(const QString &workunit);
    
    static QString stripName(const QString &fileName);
  
  private:
    void setNative(const KBSProteinPDB &native, const QStringList &workunits);
    void setFASTA(const KBSProteinFASTA &fasta, const QStringList &workunits);
    void setPsiPredSS2(const KBSRosettaPsiPredSS2 &psipred_ss2, const QStringList &workunits);
    void setV13(const KBSRosettaV13 &v1_3, unsigned index, const QStringList &workunits);
    void setOut(const KBSRosettaOut &out, const QStringList &workunits);
  
  private slots:
    void removeWorkunits(const QStringList &workunits);
    void logResults(const QStringList &results);
    void updateFile(const QString &fileName);
  
  protected:
    QHash<QString,KBSRosettaResult*> m_rosetta;
  
  private:
    QStringList m_start;
};

#endif
