/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSSETICALIBRATOR_H
#define KBSSETICALIBRATOR_H

#include <QHash>
#include <QMap>

#include <KDE/KSharedConfig>
#include <KDE/KUrl>

#include <kbsseti_export.h>

enum KBSSETICalibrationGroup {LowAR, MediumAR, HighAR};

struct KBSSETI_EXPORT KBSSETICalibration
{
  QMap<double,double> map[HighAR+1];
};

typedef QMap<double,double> KBSSETICalibrationLog;

class KBSSETITaskMonitor;

class KBSSETI_EXPORT KBSSETICalibrator : public QObject
{
  Q_OBJECT
  public:
    static KBSSETICalibrator *self();
    virtual ~KBSSETICalibrator();
    
    virtual bool isAuto() const;
    virtual void setAuto(bool set);
    
    virtual void setCalibration(const KBSSETICalibration &calibration);
    virtual const KBSSETICalibration &standardCalibration() const;
    
    virtual void resetCalibration(const KUrl &url);
    virtual double calibrate(const KUrl &url, double ar, double prog);
  
    virtual double count(const KUrl &url, unsigned group);
    virtual const KBSSETICalibration &calibration(const KUrl &url);
    
    virtual void startLog(KBSSETITaskMonitor *monitor);
    virtual void logPair(KBSSETITaskMonitor *monitor, double prog, double cpu);
    virtual void endLog(KBSSETITaskMonitor *monitor, double ar);
  
    virtual void readConfig(KSharedConfigPtr config);
    virtual void writeConfig(KSharedConfigPtr config);
  
  signals:
    void calibrationUpdated();
  
  protected:
    KBSSETICalibrator(QObject *parent=0);
    
    virtual QString id(const KUrl &url) const;
    virtual unsigned computeGroup(double ar) const;
    virtual const KBSSETICalibration &autoCalibration(const QString &id);
  
  private:
    void setupStandardCalibration();
  
  protected:
    bool m_auto;
    KBSSETICalibration m_entered, m_standard;
    QHash<QString,KBSSETICalibration> m_computed;
    QHash<QString,double> m_count[HighAR+1];
    QHash<KBSSETITaskMonitor*,KBSSETICalibrationLog*> m_logs;
  
  private:
    static KBSSETICalibrator *s_self;
};

#endif
