/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSSIMAPDATA_H
#define KBSSIMAPDATA_H

#include <QDataStream>
#include <QDateTime>
#include <QDomElement>
#include <QHash>

#include <kbssimap_export.h>

struct KBSSIMAP_EXPORT KBSSIMAPParams
{
  QHash<QString,QString> common;
  QString algorithm;
  struct {
    QString type,
            compression,
            schema;
    bool include_alignment;
  } output;
  
  bool parse(const QDomElement &node);
};

struct KBSSIMAP_EXPORT KBSSIMAPWorkunitHeader
{
  QString name;
  KBSSIMAPParams params;
  
  bool parse(const QDomElement &node);
};

struct KBSSIMAP_EXPORT KBSSIMAPTaxonomy
{
  QHash<unsigned,QString> taxonomy_node;
  QHash<unsigned,unsigned> sequence;
  
  bool parse(const QDomElement &node);
};

struct KBSSIMAP_EXPORT KBSSIMAPScreensaver
{
  KBSSIMAPTaxonomy taxonomy;
  struct {
    unsigned first_sequence,
             last_sequence,
             total_sequences;
  } queries, database;
  
  bool parse(const QDomElement &node);
};

struct KBSSIMAP_EXPORT KBSSIMAPData
{
  struct {
    QHash<unsigned,QString> sequence;
  } testset, database, queries;
  
  QString sequence(unsigned id) const;
  
  bool parse(const QDomElement &node);
};

struct KBSSIMAP_EXPORT KBSSIMAPWorkunit
{
  KBSSIMAPWorkunitHeader workunit_header;
  KBSSIMAPScreensaver screensaver;
  KBSSIMAPData data;
  
  bool parse(const QDomElement &node);
};

struct KBSSIMAP_EXPORT KBSSIMAPMatch
{
  unsigned sequence_id[2],    // query (0) and database (1) sequence id
           optscore;          // OPT score
  unsigned short identity,    // identity (percentage multiplied by 10)
                 similarity;  // similarity (percentage multiplied by 10)
  unsigned short overlap,     // overlap length
                 start[2],    // start position of the overlap in query (0) and database(1) sequence
                 end[2];      // end position of the overlap in query(0) and database(1) sequence
  
  bool parse(QDataStream &stream); 
};

struct KBSSIMAP_EXPORT KBSSIMAPResult
{
  KBSSIMAPWorkunit workunit;
  QList<KBSSIMAPMatch> matches;
};

#endif
