/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSSIMAPMONITOR_H
#define KBSSIMAPMONITOR_H

#include <QHash>

#include <kbsprojectmonitor.h>

#include <kbssimapdata.h>

class KBSSIMAPMonitor : public KBSProjectMonitor
{
  Q_OBJECT
  public:
    KBSSIMAPMonitor(const QString &project, KBSBOINCMonitor *parent);
    virtual ~KBSSIMAPMonitor();
    
    virtual const KBSSIMAPResult *simapResult(const QString &workunit);

  protected:
    virtual bool parseable(const QString &openName) const;
    virtual bool parseFile(KBSFileInfo *file, const QString &fileName);
    
    virtual KBSSIMAPResult *mkSIMAPResult(const QString &workunit);
    
    static QString parseFileName(const QString &fileName);
    static QString formatFileName(const QString &result);
  
  private slots:
    void addResults(const QStringList &results);
    void removeResults(const QStringList &results);
    void activateResult(unsigned task, const QString &result, bool active);
  
  private:
    bool parseWorkunitDocument(const QDomDocument &document, KBSSIMAPWorkunit &wu);
    
    void setWorkunit(const KBSSIMAPWorkunit &wu, const QStringList &workunits);
    void setMatches(const QList<KBSSIMAPMatch> &matches, const QStringList &workunits);
  
  private slots:
    void updateFile(const QString &fileName);
  
  protected:
    QHash<QString,KBSSIMAPResult*> m_simap;
};

#endif
