/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSTREE_H
#define KBSTREE_H

#include <QHash>
#include <QIcon>
#include <QPixmap>
#include <QStringList>

#include <kbscore_export.h>

KBSCORE_EXPORT QPixmap CompositePixmap(const QStringList &icons, int size=0);
KBSCORE_EXPORT QIcon CompositeIcon(const QStringList &icons);

typedef QList<unsigned> KBSTreePath;

class KBSTreeNode;

class KBSCORE_EXPORT KBSTreeNode : public QObject
{
  Q_OBJECT
  public:
    KBSTreeNode(QObject *parent=0);
    virtual ~KBSTreeNode();
    
    virtual unsigned children() const;
    virtual KBSTreeNode *child(int index) const;
    virtual KBSTreeNode *child(const QString &name) const;
    
    virtual int childIndex(const KBSTreeNode *child) const;
    virtual int childIndex(const QString &name) const;
    
    virtual bool isRoot() const;
    virtual KBSTreeNode *root();
    virtual bool isAncestor(const KBSTreeNode *node) const;
    virtual KBSTreeNode *findAncestor(const QString &className);
    
    virtual KBSTreePath path() const;
    virtual KBSTreeNode *descendant(const KBSTreePath &path);
    
    virtual unsigned type() const;
    virtual QString name() const;
    virtual QStringList icons() const;
  
  signals:
    void nodeChanged(KBSTreeNode *node);
    void childInserted(KBSTreeNode *node);
    void childRemoved(KBSTreeNode *node);
  
  protected:
    virtual void insertChild(KBSTreeNode *node);
    virtual void removeChild(KBSTreeNode *node, bool free=true);
    virtual void removeChild(unsigned index, bool free=true);
    virtual void removeAllChildren(bool free=true);
  
  protected:
    QList<KBSTreeNode*> m_children;
};

class KBSCORE_EXPORT KBSNamedPath : public QStringList
{
  public:
    KBSNamedPath();
    KBSNamedPath(const QString &string);
    KBSNamedPath(const KBSTreeNode *root, const KBSTreePath &path);
    virtual ~KBSNamedPath();
    
    virtual QString toString() const;
    virtual KBSTreePath toPath(const KBSTreeNode *root, bool *error=0) const;
    
    static KBSNamedPath fromString(const QString &string);
    static KBSNamedPath fromPath(const KBSTreeNode *root, const KBSTreePath &path);
};

#endif
