/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSWCGMONITOR_H
#define KBSWCGMONITOR_H

#include <QHash>

#include <kbsrosettaprojectmonitor.h>

#include <kbswcgdata.h>

class KBSMoleculeLog;

class KBSWCGMonitor : public KBSRosettaProjectMonitor
{
  Q_OBJECT
  public:
    KBSWCGMonitor(const QString &project, KBSBOINCMonitor *parent);
    virtual ~KBSWCGMonitor();
    
    virtual const KBSFAAHResult *faahResult(const QString &workunit);
    
    virtual KBSMoleculeLog *moleculeLog() const;
  
  protected:
    virtual bool parseable(const QString &openName) const;
    virtual bool parseFile(KBSFileInfo *file, const QString &fileName);
    
    virtual KBSFAAHResult *mkFAAHResult(const QString &workunit);
  
  private:
    bool parseAutoDockDocument(const QDomDocument &document, KBSFAAHAutoDock &autodock);
    
    void setDPF(const KBSFAAHParameterFile &dpf, const QStringList &workunits);
    void setGPF(const KBSFAAHParameterFile &dpf, const QStringList &workunits);
    void setPDB(const KBSProteinPDB &pdb, const QString &name, const QStringList &workunits);
    void setAutoDock(const KBSFAAHAutoDock &autodock, const QStringList &workunits);
  
  private slots:
    void removeWorkunits(const QStringList &workunits);
    void updateFile(const QString &fileName);
  
  protected:
    QHash<QString,KBSFAAHResult*> m_faah;
  
  private:
    QStringList m_start;
};

#endif
